using DbContextLib.Identity;
using Entities;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.UI.Services;
using Microsoft.AspNetCore.Mvc.ApiExplorer;
using Microsoft.AspNetCore.Mvc.Versioning;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Options;
using Microsoft.OpenApi.Models;
using Model.Manager;
using Model.Service;

using StubAPI;
using Swashbuckle.AspNetCore.SwaggerGen;


namespace HeartTrackAPI;

public class AppBootstrap(IConfiguration configuration)
{
    private IConfiguration Configuration { get; } = configuration;

    public void ConfigureServices(IServiceCollection services)
    {
        services.AddControllers();
        services.AddEndpointsApiExplorer();
        //services.AddTransient<IConfigureOptions<SwaggerGenOptions>, ConfigureSwaggerOption>();
        // include Xml comment
        // addsecurityRequiment
        // securityDef
        services.AddSwaggerGen();
        AddHeartTrackContextServices(services);
        AddModelService(services);
        AddIdentityServices(services);
        AddApiVersioning(services);
        AddSwagger(services);
        
        services.AddHealthChecks();

    }

    private void AddHeartTrackContextServices(IServiceCollection services)
    {
        var connectionString = Configuration.GetConnectionString("HeartTrackAuthConnection"); 
        if (string.IsNullOrWhiteSpace(connectionString))
        {
            throw new InvalidOperationException("The connection string for the database is not set.");
        }
        else
        {
            services.AddDbContext<AuthDbContext>(options => options.UseInMemoryDatabase("AuthDb"));
// builder.Services.AddDbContext<HeartTrackContext>(options => options.UseSqlite(connectionString));
            services.AddSingleton<IDataManager, StubData>();
        }
       
    }
    private void AddModelService(IServiceCollection services)
    {
        services.AddSingleton<IDataManager, StubData>();
        services.AddTransient<IActivityManager, ActivityManager>();
    }

    private void AddIdentityServices(IServiceCollection services)
    {
//        services.AddTransient<IEmailSender<AthleteEntity>, EmailSender>();
        services.AddAuthorization();

        services.AddIdentityApiEndpoints<IdentityUser>()
            .AddEntityFrameworkStores<AuthDbContext>();
        //services.AddIdentity<AthleteEntity, IdentityRole>()
          //  .AddEntityFrameworkStores<AuthDbContext>().AddDefaultTokenProviders();
    }
    
    private void AddApiVersioning(IServiceCollection services)
    {
        
        services.AddApiVersioning(opt =>
        {
            opt.ReportApiVersions = true;
            opt.AssumeDefaultVersionWhenUnspecified = true;
            opt.DefaultApiVersion = new Microsoft.AspNetCore.Mvc.ApiVersion(1, 0);
//            options.ApiVersionReader = new HeaderApiVersionReader("api-version");

            opt.ApiVersionReader = ApiVersionReader.Combine(new UrlSegmentApiVersionReader(),
                new HeaderApiVersionReader("x-api-version"),
                new MediaTypeApiVersionReader("x-api-version"));
        });
       
    }
    private void AddSwagger(IServiceCollection services)
    {
        services.AddSwaggerGen(options =>
        {
            options.SwaggerDoc("v1", new OpenApiInfo { Title = "HeartTrackAPI", Version = "v1" });
            options.SwaggerDoc("v2", new OpenApiInfo { Title = "HeartTrackAPI", Version = "v2" });
        });

        services.AddVersionedApiExplorer(setup =>
        {
            setup.GroupNameFormat = "'v'VVV";
            setup.SubstituteApiVersionInUrl = true;
        });
    }

    public void Configure(WebApplication app, IWebHostEnvironment env)
    {
        app.UseHttpsRedirection();

        app.MapControllers();
        app.UseAuthorization();
        app.MapIdentityApi<IdentityUser>();

        app.MapHealthChecks("/health");
        
        // Configure the HTTP request pipeline.
        if (true)
        {
            var apiVersionDescriptionProvider = app.Services.GetRequiredService<IApiVersionDescriptionProvider>();
            app.UseSwagger();
            app.UseSwaggerUI();
            app.MapSwagger();
            app.UseSwaggerUI(options =>
            {
                foreach (var description in apiVersionDescriptionProvider.ApiVersionDescriptions)

                //foreach (var description in apiVersionDescriptionProvider)
                {
                    options.SwaggerEndpoint($"/swagger/{description.GroupName}/swagger.json",
                        description.GroupName.ToUpperInvariant());
                }
            });

        }

        
    }
}
